package com.gic.haoban.manage.service.dao.mapper.contact;

import com.gic.haoban.manage.api.qdto.contact.ContactOrderSearchQDTO;
import com.gic.haoban.manage.service.entity.contact.TabContactOrder;
import org.apache.ibatis.annotations.Param;

import java.util.List;

/**
 * 建联转化订单(TabContactOrder)表数据库访问层
 *
 * @author TONGYI Lingma
 * @since 2024-09-27 16:09:38
 */
public interface TabContactOrderMapper {

    /**
     * 通过ID查询单条数据
     *
     * @param id 主键
     * @return 实例对象
     */
    TabContactOrder queryById(Long id);

    /**
     * 根据订单id查询
     * @param orderId
     * @return
     */
    TabContactOrder queryByOrderId(String orderId);

    /**
     * 通过ID列表查询多条数据
     * @param ids
     * @return
     */
    List<TabContactOrder> queryByIds(@Param("ids") List<Long> ids);

    /**
     * 查询指定行数据
     * @param enterpriseId
     * @param search
     * @return
     */
    List<TabContactOrder> queryContactOrder(@Param("enterpriseId")String enterpriseId, @Param("search") ContactOrderSearchQDTO search);

    /**
     * 统计总行数
     *
     * @param tabContactOrder 查询条件
     * @return 总行数
     */
    long count(TabContactOrder tabContactOrder);

    /**
     * 新增数据
     *
     * @param tabContactOrder 实例对象
     * @return 影响行数
     */
    int insert(TabContactOrder tabContactOrder);

    /**
     * 修改数据
     *
     * @param tabContactOrder 实例对象
     * @return 影响行数
     */
    int update(TabContactOrder tabContactOrder);

    /**
     * 通过主键删除数据
     *
     * @param id 主键
     * @return 影响行数
     */
    int deleteById(@Param("id") Long id);
}